class CustomerController < ApplicationController
  layout "admin"

  # Make sure that the user has logged in before they can take any action
  before_filter :authorize

  def index
    render :action => 'index'
  end

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  def list
    @customer_pages, @customers = paginate :customers, :per_page => 10
  end

  def show
    @customer = Customer.find(params[:id])
  end

  def new
    @customer = Customer.new
  end

  def create
    @customer = Customer.new(params[:customer])
    if @customer.save
      flash[:notice] = 'Customer was successfully created.'
      redirect_to :action => 'list'
    else
      render :action => 'new'
    end
  end

  def edit
    @customer = Customer.find(params[:id])
  end

  def update
    @customer = Customer.find(params[:id])
    if @customer.update_attributes(params[:customer])
      flash[:notice] = 'Customer was successfully updated.'
      redirect_to :action => 'show', :id => @customer
    else
      render :action => 'edit'
    end
  end

  def destroy
    Customer.find(params[:id]).destroy
    redirect_to :action => 'list'
  end

  def search
    if request.post?
      @query = params[:q]
      @customers = Customer.find(:all, :conditions => ["name like ?", @query[0]+"%"])
      render :action => 'searchresults'
    else
      render :action => 'search'
    end
  end
end
